@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 1997-2007, International Business Machines
+*   Copyright (C) 1997-2009, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -16,6 +16,9 @@
 ******************************************************************************
 */
 
+#ifndef _PLATFORM_H
+#define _PLATFORM_H
+
 /**
  * \file 
  * \brief Basic types for the platform 
@@ -26,18 +29,25 @@
 #define U_BSD
 #endif
 
-/* Define whether inttypes.h is available */
+/**
+ * \def U_HAVE_DIRENT_H
+ * Define whether dirent.h is available */
+#ifndef U_HAVE_DIRENT_H
+#define U_HAVE_DIRENT_H 1
+#endif
+
+/** Define whether inttypes.h is available */
 #ifndef U_HAVE_INTTYPES_H
 #define U_HAVE_INTTYPES_H 1
 #endif
 
-/*
+/**
  * Define what support for C++ streams is available.
- *     If U_IOSTREAM_SOURCE is set to 199711, then <iostream> is available
+ *     If U_IOSTREAM_SOURCE is set to 199711, then &lt;iostream&gt; is available
  * (1997711 is the date the ISO/IEC C++ FDIS was published), and then
  * one should qualify streams using the std namespace in ICU header
  * files.
- *     If U_IOSTREAM_SOURCE is set to 198506, then <iostream.h> is
+ *     If U_IOSTREAM_SOURCE is set to 198506, then &lt;iostream.h&gt; is
  * available instead (198506 is the date when Stroustrup published
  * "An Extensible I/O Facility for C++" at the summer USENIX conference).
  *     If U_IOSTREAM_SOURCE is 0, then C++ streams are not available and
@@ -49,7 +59,19 @@
 #define U_IOSTREAM_SOURCE 199711
 #endif
 
-/* Determines whether specific types are available */
+/**
+ * \def U_HAVE_STD_STRING
+ * Define whether the standard C++ (STL) <string> header is available.
+ * For platforms that do not use platform.h and do not define this constant
+ * in their platform-specific headers, std_string.h defaults
+ * U_HAVE_STD_STRING to 1.
+ * @draft ICU 4.2
+ */
+#ifndef U_HAVE_STD_STRING
+#define U_HAVE_STD_STRING 1
+#endif
+
+/** @{ Determines whether specific types are available */
 #ifndef U_HAVE_INT8_T
 #define U_HAVE_INT8_T 1
 #endif
@@ -82,8 +104,10 @@
 #define U_HAVE_UINT64_T 1
 #endif
 
+/** @} */
+
 /*===========================================================================*/
-/* Generic data types                                                        */
+/** @{ Generic data types                                                        */
 /*===========================================================================*/
 
 #include <sys/types.h>
@@ -148,8 +172,10 @@ typedef unsigned int uint32_t;
 
 #endif
 
+/** @} */
+
 /*===========================================================================*/
-/* Compiler and environment features                                         */
+/** @{ Compiler and environment features                                         */
 /*===========================================================================*/
 
 /* Define whether namespace is supported */
@@ -163,7 +189,7 @@ typedef unsigned int uint32_t;
 #if defined(BYTE_ORDER) && defined(BIG_ENDIAN)
 #define U_IS_BIG_ENDIAN (BYTE_ORDER == BIG_ENDIAN)
 #else
-#define U_IS_BIG_ENDIAN 1
+#define U_IS_BIG_ENDIAN 0
 #endif
 
 /* 1 or 0 to enable or disable threads.  If undefined, default is: enable threads. */
@@ -212,24 +238,38 @@ typedef unsigned int uint32_t;
 #define U_LIB_SUFFIX_C_NAME 
 #define U_LIB_SUFFIX_C_NAME_STRING ""
 
+/** @} */
+
 /*===========================================================================*/
-/* Character data types                                                      */
+/** @{ Character data types                                                      */
 /*===========================================================================*/
 
 #if ((defined(OS390) && (!defined(__CHARSET_LIB) || !__CHARSET_LIB))) || defined(OS400)
 #   define U_CHARSET_FAMILY 1
 #endif
 
+/** @} */
+
 /*===========================================================================*/
-/* Information about wchar support                                           */
+/** @{ Information about wchar support                                           */
 /*===========================================================================*/
 
+#ifndef U_HAVE_WCHAR_H
 #define U_HAVE_WCHAR_H      1
-#define U_SIZEOF_WCHAR_T    4
+#endif
 
+#ifndef U_SIZEOF_WCHAR_T
+#define U_SIZEOF_WCHAR_T    4
+#endif
+
+#ifndef U_HAVE_WCSCPY
 #define U_HAVE_WCSCPY       1
+#endif
+
+/** @} */
 
 /**
+ * @{
  * \def U_DECLARE_UTF16
  * Do not use this macro. Use the UNICODE_STRING or U_STRING_DECL macros
  * instead.
@@ -250,12 +290,19 @@ typedef unsigned int uint32_t;
 #endif
 #endif
 
+/** @} */
+
 /*===========================================================================*/
-/* Information about POSIX support                                           */
+/** @{ Information about POSIX support                                           */
 /*===========================================================================*/
 
+#ifndef U_HAVE_NL_LANGINFO_CODESET
 #define U_HAVE_NL_LANGINFO_CODESET  1
+#endif
+
+#ifndef U_NL_LANGINFO_CODESET
 #define U_NL_LANGINFO_CODESET       CODESET
+#endif
 
 #if 1
 #define U_TZSET         tzset
@@ -270,8 +317,10 @@ typedef unsigned int uint32_t;
 #define U_HAVE_MMAP     1
 #define U_HAVE_POPEN    1
 
+/** @} */
+
 /*===========================================================================*/
-/* Symbol import-export control                                              */
+/** @{ Symbol import-export control                                              */
 /*===========================================================================*/
 
 #if 0
@@ -295,22 +344,38 @@ typedef unsigned int uint32_t;
 #define U_IMPORT 
 #endif
 
+/* @} */
+
 /*===========================================================================*/
-/* Code alignment and C function inlining                                    */
+/** @{ Code alignment and C function inlining                                    */
 /*===========================================================================*/
 
 #ifndef U_INLINE
 #   ifdef __cplusplus
 #       define U_INLINE inline
 #   else
-#       define U_INLINE inline
+#       define U_INLINE __inline__
 #   endif
 #endif
 
+#ifndef U_ALIGN_CODE
 #define U_ALIGN_CODE(n) 
+#endif
+
+/** @} */
 
 /*===========================================================================*/
-/* Programs used by ICU code                                                 */
+/** @{ Programs used by ICU code                                                 */
 /*===========================================================================*/
 
+/**
+ * \def U_MAKE
+ * What program to execute to run 'make'
+ */
+#ifndef U_MAKE
 #define U_MAKE  "/usr/local/bin/gmake"
+#endif
+
+/** @} */
+
+#endif
